#ifndef _REQUEST_PARSER_H
#define _REQUEST_PARSER_H

#include <libxml/parser.h>

#include "libs/buffer.h"

struct request;

struct request_parser_handler {
	void (*on_start_textforexplanation)(struct request *);
	void (*on_start_doccategory)(struct request *);
	void (*on_start_elementcategory)(struct request *);
	void (*on_start_siteunit)(struct request *);
	void (*on_start_text)(struct request *);

	void (*on_end_textforexplanation)(struct request *);
	void (*on_end_doccategory)(struct request *);
	void (*on_end_elementcategory)(struct request *);
	void (*on_end_siteunit)(struct request *);
	void (*on_end_text)(struct request *);

	void (*on_characters_doccategory)(
		struct request *,
		const char *,
		unsigned int
	);
	void (*on_characters_elementcategory)(
		struct request *,
		const char *,
		unsigned int
	);
	void (*on_characters_siteunit)(
		struct request *,
		const char *,
		unsigned int
	);
	void (*on_characters_text)(
		struct request *,
		const char *,
		unsigned int
	);
};

struct request_parser {
	enum request_parser_state {
		REQUEST_PARSER_ROOT,
		REQUEST_PARSER_TEXTFOREXPLANATION,
		REQUEST_PARSER_DOCCATEGORY,
		REQUEST_PARSER_ELEMENTCATEGORY,
		REQUEST_PARSER_SITEUNIT,
		REQUEST_PARSER_TEXT,
		REQUEST_PARSER_ERROR,
		REQUEST_PARSER_DONE,
	} state;

	/* error message (NULL if none) */
	char *error;

	struct request *request;
	const struct request_parser_handler *handler;
};

extern const xmlSAXHandler request_parser_handler;

void
request_parser_init(struct request_parser *parser);

void
request_parser_deinit(struct request_parser *parser);

#endif
