#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libxml/parser.h>

#include "libs/buffer.h"
#include "libs/list.h"
#include "libs/xmalloc.h"
#include "libs/xml.h"

#include "request.h"
#include "request-parser.h"
#include "text-parser.h"

void
request_init(struct request *request)
{
	assert(request);

	buffer_init(&request->output);
}

void
request_deinit(struct request *request)
{
	assert(request);

	buffer_deinit(&request->output, 1);
}

static void
on_start_textforexplanation(struct request *request)
{
	struct buffer *output;

	output = &request->output;
	buffer_write_string(output, "<document>");
	/* XXX: Fixme, don't hardcode */
	buffer_write_string(output, "<sources>");
	buffer_write_string(output, "<thesaurus>1</thesaurus>");
	buffer_write_string(output, "<thesaurus>2</thesaurus>");
	buffer_write_string(output, "<thesaurus>3</thesaurus>");
	buffer_write_string(output, "</sources>");
}

static void
on_end_textforexplanation(struct request *request)
{
	struct buffer *output;

	output = &request->output;
	buffer_write_string(output, "</document>");
}

static void
on_start_doccategory(struct request *request)
{
	/* begin DocCategory reading */
	buffer_init(&request->DocCategory);
}

static void
on_characters_doccategory(struct request *request,
	const char *s, unsigned int l)
{
	buffer_write(&request->DocCategory, s, l);
}

static void
on_end_doccategory(struct request *request)
{
	/* end DocCategory reading */
	buffer_deinit(&request->DocCategory, 1);
}

static void
on_start_elementcategory(struct request *request)
{
	/* begin ElementCategory reading */
	buffer_init(&request->ElementCategory);
}

static void
on_characters_elementcategory(struct request *request,
	const char *s, unsigned int l)
{
	buffer_write(&request->ElementCategory, s, l);
}

static void
on_end_elementcategory(struct request *request)
{
	/* end ElementCategory reading */
	buffer_deinit(&request->ElementCategory, 1);
}

static void
on_start_siteunit(struct request *request)
{
	/* begin SiteUnit reading */
	buffer_init(&request->SiteUnit);
}

static void
on_characters_siteunit(struct request *request,
	const char *s, unsigned int l)
{
	buffer_write(&request->SiteUnit, s, l);
}

static void
on_end_siteunit(struct request *request)
{
	buffer_deinit(&request->SiteUnit, 1);
}

static void
st_on_word(struct text_parser *parser, const char *s, unsigned int l)
{
	struct request *request;
	struct buffer *output;

	request = parser->data;
	output = &request->output;

	buffer_write_string(output, "<word>");
	xml_escape_to_buffer(s, l, output);
	buffer_write_string(output, "</word>");

#if 0
	printf("\x1b[31m[\x1b[37m%.*s\x1b[31m]", l, s);
#endif
}

static void
st_on_separator(struct text_parser *parser, const char *s, unsigned int l)
{
	struct request *request;
	struct buffer *output;

	request = parser->data;
	output = &request->output;

	xml_escape_to_buffer(s, l, output);

#if 0
	printf("\x1b[32m%.*s", l, s);
#endif
}

static void
on_start_text(struct request *request)
{
	struct buffer *output;

	output = &request->output;
	buffer_write_string(output, "<text>");

	/* begin Text reading */
	text_parser_init(&request->Text);
	request->Text.data = request;
	request->Text.on_word = &st_on_word;
	request->Text.on_separator = &st_on_separator;
}

static void
on_characters_text(struct request *request, const char *s, unsigned int l)
{
	text_parser_parse(&request->Text, s, l);
}

static void
on_end_text(struct request *request)
{
	struct buffer *output;

	/* end Text reading */
	text_parser_finish(&request->Text);
	text_parser_deinit(&request->Text);

	output = &request->output;
	buffer_write_string(output, "</text>");
}

const struct request_parser_handler request_handler = {
	.on_start_textforexplanation	= &on_start_textforexplanation,
	.on_start_doccategory		= &on_start_doccategory,
	.on_start_elementcategory	= &on_start_elementcategory,
	.on_start_siteunit		= &on_start_siteunit,
	.on_start_text			= &on_start_text,

	.on_end_textforexplanation	= &on_end_textforexplanation,
	.on_end_doccategory		= &on_end_doccategory,
	.on_end_elementcategory		= &on_end_elementcategory,
	.on_end_siteunit		= &on_end_siteunit,
	.on_end_text			= &on_end_text,

	.on_characters_doccategory	= &on_characters_doccategory,
	.on_characters_elementcategory	= &on_characters_elementcategory,
	.on_characters_siteunit		= &on_characters_siteunit,
	.on_characters_text		= &on_characters_text,
};
