#ifndef _RESPONSE_PARSER_H
#define _RESPONSE_PARSER_H

#include <libxml/parser.h>

#include "libs/buffer.h"

struct response;

struct response_parser_handler {
	void (*on_start_document)(struct response *);
	void (*on_start_text)(struct response *);
	void (*on_start_word)(struct response *);
	void (*on_start_original)(struct response *);
	void (*on_start_result)(struct response *,
		int, int, int, unsigned int);
	void (*on_start_synonym)(struct response *);
	void (*on_start_term)(struct response *);
	void (*on_start_explanation)(struct response *);
	void (*on_start_thesaurus)(struct response *);

	void (*on_end_document)(struct response *);
	void (*on_end_text)(struct response *);
	void (*on_end_word)(struct response *);
	void (*on_end_original)(struct response *);
	void (*on_end_result)(struct response *);
	void (*on_end_synonym)(struct response *);
	void (*on_end_term)(struct response *);
	void (*on_end_explanation)(struct response *);
	void (*on_end_thesaurus)(struct response *);

	void (*on_characters_text)(
		struct response *,
		const char *,
		unsigned int
	);
	void (*on_characters_original)(
		struct response *,
		const char *,
		unsigned int
	);
	void (*on_characters_synonym)(
		struct response *,
		const char *,
		unsigned int
	);
	void (*on_characters_term)(
		struct response *,
		const char *,
		unsigned int
	);
	void (*on_characters_explanation)(
		struct response *,
		const char *,
		unsigned int
	);
	void (*on_characters_thesaurus)(
		struct response *,
		const char *,
		unsigned int
	);
};

struct response_parser {
	enum response_parser_state {
		RESPONSE_PARSER_ROOT,
		RESPONSE_PARSER_DOCUMENT,
		RESPONSE_PARSER_TEXT,
		RESPONSE_PARSER_WORD,
		RESPONSE_PARSER_ORIGINAL,
		RESPONSE_PARSER_RESULT,
		RESPONSE_PARSER_SYNONYM,
		RESPONSE_PARSER_TERM,
		RESPONSE_PARSER_EXPLANATION,
		RESPONSE_PARSER_THESAURUS,
		RESPONSE_PARSER_ERROR,
		RESPONSE_PARSER_DONE,
	} state;

	/* error message (NULL if none) */
	char *error;

	struct response *response;
	const struct response_parser_handler *handler;
};

extern const xmlSAXHandler response_parser_handler;

void
response_parser_init(struct response_parser *parser);

void
response_parser_deinit(struct response_parser *parser);

#endif
