#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "libs/buffer.h"

#include "text-parser.h"

void
text_parser_init(struct text_parser *parser)
{
	assert(parser);

	parser->state = TEXT_PARSER_INIT;
}

void
text_parser_deinit(struct text_parser *parser)
{
	assert(parser);

	/* buffer is deinitialized at this point */
}

void
text_parser_parse(struct text_parser *parser,
	const char *s, unsigned int l)
{
	static const char *separators =
		" \t\n"
		"!\"#$%&'()*+,-./:;<=>?";

	unsigned int i;

	assert(parser);
	assert(s);

	for(i = 0; i < l; i++) {
		switch(parser->state) {
		case TEXT_PARSER_INIT:
			if(index(separators, s[i])) {
				buffer_init(&parser->buffer);
				buffer_write_char(&parser->buffer, s[i]);
				parser->state = TEXT_PARSER_SEPARATOR;
				break;
			}

			buffer_init(&parser->buffer);
			buffer_write_char(&parser->buffer, s[i]);
			parser->state = TEXT_PARSER_WORD;
			break;
		case TEXT_PARSER_WORD:
			if(s[i] == '.') {
				parser->state = TEXT_PARSER_WORD_DOT;
				break;
			}

			if(index(separators, s[i])) {
				parser->on_word(parser,
					parser->buffer.data, parser->buffer.i);
				buffer_deinit(&parser->buffer, 1);

				buffer_init(&parser->buffer);
				buffer_write_char(&parser->buffer, s[i]);
				parser->state = TEXT_PARSER_SEPARATOR;
				break;
			}

			buffer_write_char(&parser->buffer, s[i]);
			break;
		case TEXT_PARSER_WORD_DOT:
			if(index(separators, s[i])) {
				parser->on_word(parser,
					parser->buffer.data, parser->buffer.i);
				buffer_deinit(&parser->buffer, 1);

				buffer_init(&parser->buffer);
				buffer_write_char(&parser->buffer, '.');
				buffer_write_char(&parser->buffer, s[i]);
				parser->state = TEXT_PARSER_SEPARATOR;
				break;
			}

			buffer_write_char(&parser->buffer, '.');
			buffer_write_char(&parser->buffer, s[i]);
			parser->state = TEXT_PARSER_WORD;
			break;
		case TEXT_PARSER_SEPARATOR:
			if(!index(separators, s[i])) {
				parser->on_separator(parser,
					parser->buffer.data, parser->buffer.i);
				buffer_deinit(&parser->buffer, 1);

				buffer_init(&parser->buffer);
				buffer_write_char(&parser->buffer, s[i]);
				parser->state = TEXT_PARSER_WORD;
				break;
			}

			buffer_write_char(&parser->buffer, s[i]);
			break;
		}
	}
}

void
text_parser_finish(struct text_parser *parser)
{
	switch(parser->state) {
	case TEXT_PARSER_INIT:
		/* buffer is not even initialized. */
		break;
	case TEXT_PARSER_WORD:
		if(!parser->buffer.i)
			break;

		parser->on_word(parser,
			parser->buffer.data, parser->buffer.i);
		buffer_deinit(&parser->buffer, 1);
		break;
	case TEXT_PARSER_WORD_DOT:
		if(!parser->buffer.i)
			break;

		parser->on_word(parser,
			parser->buffer.data, parser->buffer.i);
		buffer_deinit(&parser->buffer, 1);

		parser->on_separator(parser,
			".", 1);
		break;
	case TEXT_PARSER_SEPARATOR:
		if(!parser->buffer.i)
			break;

		parser->on_separator(parser,
			parser->buffer.data, parser->buffer.i);
		buffer_deinit(&parser->buffer, 1);
		break;
	}
}
