use strict;
use warnings;
use utf8;

use DBI;
use Encode;
use File::DirWalk;
use XML::DOM;

my $mj2 = DBI->connect('dbi:mysql:minjournal2', 'minjournal2', undef);
my $dw = File::DirWalk->new;

$dw->onFile(sub {
	my $filename = shift;

	return File::DirWalk::SUCCESS if($filename !~ m/html\.vnr\.html$/);

	my $parser = XML::DOM::Parser->new;
	my $doc = $parser->parsefile($filename);

	my $nodes = $doc->getElementsByTagName("meta");

	my $meta_keywords = '';
	my $meta_description = '';

	for(my $i = 0; $i < $nodes->getLength; $i++) {
		my $node = $nodes->item($i);

		next if $node->getNodeType != ELEMENT_NODE;

		$meta_keywords = Encode::encode_utf8($node->getAttribute('content')) if($node->getAttribute('name') eq 'keywords');
		$meta_description = Encode::encode_utf8($node->getAttribute('content')) if($node->getAttribute('name') eq 'description');
	}

	$meta_keywords =~ s/[\s\r\n]+/ /g;
	$meta_description =~ s/[\s\r\n]+/ /g;

	$meta_keywords =~ s/\s\xc2\xab(.+)\xc2\xbb$//;
	my $manifacturer = $1;
	my $name = $meta_keywords;
	my $description = $meta_description;

	#printf "%s: %s: %s\n", $manifacturer, $name, $description;

	for my $term (split m/, /, $name) {
		my $q = $mj2->prepare('INSERT INTO Term (thesaurus, word, explanation) '
			. 'VALUES (?, ?, ?)');
		my $rv = $q->execute(3, $term, $description);
	}

	return File::DirWalk::SUCCESS;
});

$dw->walk('preparat-register');
