#ifndef _IO_SOCKET_BUFFER_H
#define _IO_SOCKET_BUFFER_H

/* Socket buffer
 * Write: uses linked list of unwritten buffers. write() simply appends a
 *        new element to list.
 * Read:  uses read pointer in first element of unwritten buffers. If read
 *        pointer reaches the end, free the element and take the next one.
 */

struct buffer;

struct io_socket_buffer_element {
	struct io_socket_buffer_element *_prev, *_next;

	unsigned long size;
	unsigned char *data;

	unsigned long rptr;
};

struct io_socket_buffer {
	struct io_socket_buffer_element elements;
};

void
io_socket_buffer_init(struct io_socket_buffer *sb);
void
io_socket_buffer_deinit(struct io_socket_buffer *sb);

int
io_socket_buffer_is_full(struct io_socket_buffer *sb);
unsigned long int
io_socket_buffer_can_write(struct io_socket_buffer *sb);
void
io_socket_buffer_write(struct io_socket_buffer *sb,
	unsigned char *data, unsigned long size);
void
io_socket_buffer_write_dup(struct io_socket_buffer *sb,
	const unsigned char *data, unsigned long size);
void
io_socket_buffer_write_buffer(struct io_socket_buffer *sb,
	struct buffer *buffer);

int
io_socket_buffer_is_empty(struct io_socket_buffer *sb);
unsigned long int
io_socket_buffer_can_read(struct io_socket_buffer *sb);
void
io_socket_buffer_read(struct io_socket_buffer *sb, unsigned char **data);
void
io_socket_buffer_advance(struct io_socket_buffer *sb, unsigned long size);

#endif
