#ifndef _IO_SOCKET_H
#define _IO_SOCKET_H

#ifndef WIN32
#include <sys/socket.h>
#endif

#include <sys/types.h>
#include <stdint.h>

#ifdef WIN32
#include <ws2tcpip.h>
#endif

#include "libs/io-socket-buffer.h"

struct io_socket {
	struct io_socket *_prev;
	struct io_socket *_next;

	struct io_loop *io_loop;

	int fd;
	struct sockaddr addr;
	socklen_t addrlen;

	enum {
		CLIENT,
		SERVER,
	} type;

	/* Set to 1 if reading should not be done. */
	unsigned int dont_read;

	/* Set to 1 if writing should not be done. */
	unsigned int dont_write;

	unsigned int close;
	struct io_socket_buffer ibuffer, obuffer;

	void *data;
	void (*on_accept)(struct io_socket *);
	void (*on_read)(struct io_socket *);
	void (*on_write)(struct io_socket *);
	void (*on_close)(struct io_socket *);
};

void
io_socket_init(struct io_socket *socket);

void
io_socket_deinit(struct io_socket *socket);

void
io_socket_read(struct io_socket *socket);

void
io_socket_write(struct io_socket *socket);

void
io_socket_close(struct io_socket *socket);

int
io_socket_open(struct io_socket *socket, const char *filename, int flags);

int
io_socket_connect(struct io_socket *socket, uint32_t host, uint16_t port);

int
io_socket_accept(struct io_socket *socket);

int
io_socket_listen(struct io_socket *socket, unsigned short int port);

#endif
