#ifndef _LIST_H
#define _LIST_H

#define LIST_INIT(head, ...)					\
	do {							\
		(head)-> __VA_ARGS__ ## _ ## prev = (head);	\
		(head)-> __VA_ARGS__ ## _ ## next = (head);	\
	} while(0);

#define LIST_INSERT_BEFORE(head, node, ...)						\
	do {										\
		(node)-> __VA_ARGS__ ## _ ## prev = (head)-> __VA_ARGS__ ## _ ## prev;	\
		(node)-> __VA_ARGS__ ## _ ## next = (head);				\
		(head)-> __VA_ARGS__ ## _ ## prev-> __VA_ARGS__ ## _ ## next = (node);	\
		(head)-> __VA_ARGS__ ## _ ## prev = (node);				\
	} while(0);

#define LIST_INSERT_AFTER(head, node, ...)						\
	do {										\
		(node)-> __VA_ARGS__ ## _ ## prev = (head);				\
		(node)-> __VA_ARGS__ ## _ ## next = (head)-> __VA_ARGS__ ## _ ## next;	\
		(head)-> __VA_ARGS__ ## _ ## next-> __VA_ARGS__ ## _ ## prev = (node);	\
		(head)-> __VA_ARGS__ ## _ ## next = (node);				\
	} while(0);

/* It is asserted that once LIST_REMOVE(node) has been called, the repeated
 * statement will have no effect (unless, of course, the node has been
 * inserted again). */
#define LIST_REMOVE(node, ...)												\
	do {														\
		(node)-> __VA_ARGS__ ## _ ## prev-> __VA_ARGS__ ## _ ## next = (node)-> __VA_ARGS__ ## _ ## next;	\
		(node)-> __VA_ARGS__ ## _ ## next-> __VA_ARGS__ ## _ ## prev = (node)-> __VA_ARGS__ ## _ ## prev;	\
		(node)-> __VA_ARGS__ ## _ ## prev = (node);								\
		(node)-> __VA_ARGS__ ## _ ## next = (node);								\
	} while(0);

#define LIST_IS_EMPTY(head, ...) \
	((head)-> __VA_ARGS__ ## _ ## next == (head))

/* Always check LIST_IS_EMPTY() before calling this. */
#define LIST_FIRST(head, ...) \
	((head) -> __VA_ARGS__ ## _ ## next)

/* Always check LIST_IS_EMPTY() before calling this. */
#define LIST_LAST(head, ...) \
	((head) -> __VA_ARGS__ ## _ ## prev)

#define LIST_FOR(head, node, ...) \
	for((node) = (head)-> __VA_ARGS__ ## _ ## next; \
		(node) != (head); \
		(node) = (node)-> __VA_ARGS__ ## _ ## next)

#define LIST_FOR_REVERSE(head, node, ...) \
	for((node) = (head)-> __VA_ARGS__ ## _ ## prev; \
		(node) != (head); \
		(node) = (node)-> __VA_ARGS__ ## _ ## prev)

#define LIST_FOR_SAFE(head, node, temp, ...) \
	for((node) = (head)-> __VA_ARGS__ ## _ ## next, \
		(temp) = (node)-> __VA_ARGS__ ## _ ## next; \
		(node) != (head); \
		(node) = (temp), \
		(temp) = (node)-> __VA_ARGS__ ## _ ## next)

#define LIST_FOR_REVERSE_SAFE(head, node, temp, ...) \
	for((node) = (head)-> __VA_ARGS__ ## _ ## prev, \
		(temp) = (node)-> __VA_ARGS__ ## _ ## prev; \
		(node) != (head); \
		(node) = (temp), \
		(temp) = (node)-> __VA_ARGS__ ## _ ## prev)

#endif
