#include <stdint.h>
#include <stdlib.h>

#include "libs/xdr-array.h"
#include "libs/xmalloc.h"

struct write_buffer;
void write_buffer_write_u8(struct write_buffer *wb, uint8_t data);
void write_buffer_write_u16(struct write_buffer *wb, uint16_t data);

struct read_buffer;
uint8_t read_buffer_read_u8(struct read_buffer *rb);
uint16_t read_buffer_read_u16(struct read_buffer *rb);

void
xdr_array_serialize(struct write_buffer *wb, struct xdr_array *a)
{
	unsigned long int i;

	write_buffer_write_u8(wb, a->type);
	write_buffer_write_u16(wb, a->n);

	switch(a->type) {
	case ARRAY:
		for(i = 0; i < a->n; i++)
			xdr_array_serialize(wb, a->e.e_array[i]);

		free(a->e.e_array);
		break;
	case CHAR:
		for(i = 0; i < a->n; i++)
			write_buffer_write_u8(wb, a->e.e_char[i]);

		free(a->e.e_char);
		break;
	}

	free(a);
}

struct xdr_array *
xdr_array_deserialize(struct read_buffer *rb)
{
	struct xdr_array *a;
	unsigned long int i;

	a = xmalloc(sizeof(struct xdr_array));
	a->type = read_buffer_read_u8(rb);
	a->n = read_buffer_read_u16(rb);

	switch(a->type) {
	case ARRAY:
		a->e.e_array = xcalloc(a->n, sizeof(*a->e.e_array));

		for(i = 0; i < a->n; i++)
			a->e.e_array[i] = xdr_array_deserialize(rb);

		break;
	case CHAR:
		a->e.e_char = xcalloc(a->n, sizeof(*a->e.e_char));

		for(i = 0; i < a->n; i++)
			a->e.e_char[i] = read_buffer_read_u8(rb);

		break;
	}

	return a;
}
