#include <assert.h>

#include "libs/buffer.h"
#include "libs/xmalloc.h"

void
xml_escape_to_buffer(const char *s, unsigned int l,
	struct buffer *buffer)
{
	unsigned int i;

	assert(s);
	assert(l);
	assert(buffer);

	for(i = 0; i < l; i++) {
		switch(s[i]) {
		case '&':
			buffer_write_string(buffer, "&amp;");
			break;
		case '<':
			buffer_write_string(buffer, "&lt;");
			break;
		case '>':
			buffer_write_string(buffer, "&gt;");
			break;
		case '\'':
			buffer_write_string(buffer, "&apos;");
			break;
		case '\"':
			buffer_write_string(buffer, "&quot;");
			break;
		default:
			buffer_write_char(buffer, s[i]);
			break;
		}
	}
}

void
xml_escape_buffer_to_buffer(struct buffer *in, struct buffer *out)
{
	xml_escape_to_buffer(in->data, in->i, out);
}

void
xml_escape(const char *s, unsigned int l,
	char **escaped, unsigned int *escaped_length)
{
	struct buffer buffer;

	assert(escaped);
	assert(escaped_length);

	buffer_init(&buffer);
	xml_escape_to_buffer(s, l, &buffer);
	buffer_move(&buffer, escaped, escaped_length);
	buffer_deinit(&buffer, 0);
}
