#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libxml/parser.h>

#include "buffer.h"

struct request_state {
	enum {
		ROOT,
		ROOT_REQUEST,
		ROOT_REQUEST_WORD,
	} state;

	struct buffer word;
};

static void
st_request_state_init(struct request_state *state)
{
	state->state = ROOT;
}

static void
st_startElement(void *data, const xmlChar *name, const xmlChar **attrs)
{
	struct request_state *state;

	assert(data);

	state = data;
	switch(state->state) {
	case ROOT:
		if(!strcmp((char *) name, "request")) {
			state->state = ROOT_REQUEST;
			break;
		}

		/* error */
		break;
	case ROOT_REQUEST:
		if(!strcmp((char *) name, "word")) {
			buffer_init(&state->word);

			state->state = ROOT_REQUEST_WORD;
			break;
		}

		/* error */
		break;
	case ROOT_REQUEST_WORD:
		/* error */
		break;
	}
}

static void
st_endElement(void *data, const xmlChar *name)
{
	struct request_state *state;

	assert(data);

	state = data;
	switch(state->state) {
	case ROOT:
		/* error */
		break;
	case ROOT_REQUEST:
		if(!strcmp((char *) name, "request")) {
			state->state = ROOT;
			break;
		}

		/* error */
		break;
	case ROOT_REQUEST_WORD:
		if(!strcmp((char *) name, "word")) {
			/* XXX: Handle state->word buffer here. */
			printf("looking up word... <%.*s>\n",
				state->word.i, state->word.data);

			buffer_deinit(&state->word);
			
			state->state = ROOT_REQUEST;
			break;
		}

		/* error */
		break;
	}
}

static void
st_characters(void *data, const xmlChar *ch, int len)
{
	struct request_state *state;

	assert(data);

	state = data;
	switch(state->state) {
	case ROOT:
		/* ignore */
		break;
	case ROOT_REQUEST:
		/* ignore */
		break;
	case ROOT_REQUEST_WORD:
		buffer_write(&state->word, (char *) ch, len);
		break;
	}
}

static xmlSAXHandler handler = {
	.startElement = &st_startElement,
	.endElement = &st_endElement,
	.characters = &st_characters,
};

int
main(int argc, char *argv[])
{
	int ret;
	xmlParserCtxtPtr parserCtxt;
	struct request_state state;

	st_request_state_init(&state);

	parserCtxt = xmlCreatePushParserCtxt(&handler, &state, NULL, 0, NULL);
	if(!parserCtxt) {
		fprintf(stderr, "error\n");
		exit(1);
	}

	ret = xmlParseChunk(parserCtxt, argv[1], strlen(argv[1]), 0);
	if(ret) {
		fprintf(stderr, "error\n");
		exit(1);
	}

	return 0;
}
