#include <stdio.h>
#include <stdlib.h>

#include "libs/list.h"
#include "category.h"
#include "category-hashtable.h"

void
category_hashtable_init(struct category_hashtable *table)
{
	unsigned int i;

	for(i = 0; i < 256; i++)
		LIST_INIT(&table->buckets[i]);
}

void
category_hashtable_deinit(struct category_hashtable *table)
{
	unsigned int i;
	struct category *category;
	struct category *category_temp;

	for(i = 0; i < 256; i++) {
		LIST_FOR_SAFE(&table->buckets[i], category, category_temp) {
			LIST_REMOVE(category);
			category_deinit(category);
			free(category);
		}
	}
}

static unsigned int
st_hash(unsigned long int id)
{
	return id % 256;
}

struct category *
category_hashtable_get(struct category_hashtable *table,
	unsigned long int id)
{
	unsigned int i;
	struct category *bucket;
	struct category *category;

	i = st_hash(id);
	bucket = &table->buckets[i];

	LIST_FOR(bucket, category) {
		if(category->id == id)
			return category;
	}

	return NULL;
}

void
category_hashtable_put(struct category_hashtable *table,
	struct category *category)
{
	unsigned int i;
	struct category *bucket;

	i = st_hash(category->id);
	bucket = &table->buckets[i];

	LIST_INSERT_BEFORE(bucket, category);
}
