#include <assert.h>
#include <stdlib.h>

#include <libxml/parser.h>

#include "libs/io-socket.h"

#include "connection.h"
#include "request-parser.h"

void
connection_init(struct connection *c)
{
	assert(c);

	/* Grr... libxml's fault for not taking consts... */
	c->xml_parser = xmlCreatePushParserCtxt(&request_parser_handler,
		&c->request_parser, NULL, 0, NULL);
	if(!c->xml_parser) {
		/* XXX: Graceful exit (close socket?) */
		fprintf(stderr, "Could not create XML parser context!\n");
		exit(1);
	}

	request_parser_init(&c->request_parser);
	c->request_parser.request = &c->request;
	c->request_parser.handler = &request_handler;

	request_init(&c->request);

	c->lookups = 0;
	c->results = 0;
}

void
connection_deinit(struct connection *c)
{
	assert(c);

	/* XXX: Do we also have to free c->parser->myDoc? (SAX parser) */
	xmlFreeParserCtxt(c->xml_parser);
	request_deinit(&c->request);
	request_parser_deinit(&c->request_parser);
}

void
connection_handle(struct connection *c,
	const unsigned char *data, unsigned long int n)
{
	struct request *request;

	assert(c);
	assert(data);

	request = &c->request;

	xmlParseChunk(c->xml_parser, (const char *) data, n, 0);

	if(c->request_parser.state == REQUEST_PARSER_ERROR) {
		/* Error */
		printf("error: %d: %s\n",
			c->xml_parser->input->line,
			c->request_parser.error
				? c->request_parser.error : "(unknown)");
	}

	/* The parser created some output which must be written to the
	 * socket. */
	io_socket_buffer_write_buffer(&c->socket->obuffer, &request->output);
}
