#include <assert.h>
#include <string.h>

#include "edit-distance.h"

static unsigned int
st_min3(unsigned int a, unsigned int b, unsigned int c)
{
	unsigned int n;

	n = a;
	if(b < n)
		n = b;
	if(c < n)
		n = c;

	return n;
}

static unsigned int
st_min2(unsigned int a, unsigned int b)
{
	return a < b ? a : b;
}

static unsigned int
st_max2(unsigned int a, unsigned int b)
{
	return a > b ? a : b;
}

static unsigned int
st_longest_common_prefix(const char *s1, unsigned int l1,
	const char *s2, unsigned int l2, unsigned int n)
{
	unsigned int i;

	for(i = 0; i < n; i++) {
		if(s1[i] != s2[i])
			return i;
	}

	return i;
}

static unsigned int
st_longest_common_suffix(const char *s1, unsigned int l1,
	const char *s2, unsigned int l2, unsigned int n)
{
	unsigned int i;
	unsigned int i1;
	unsigned int i2;

	i1 = l1 - 1;
	i2 = l2 - 1;
	for(i = 0; i < n; i++) {
		if(s1[i1] != s2[i2])
			return i;

		i1--;
		i2--;
	}

	return i;
}

/* Edit distance. An approximation of Levenshtein distance in O(M+N) time
 * where M and N are the word lengths. The distance returned may be larger
 * than the Levenshtein distance for the same arguments, but never smaller.
 */
unsigned int
edit_distance(const char *s1, unsigned int l1,
	const char *s2, unsigned int l2)
{
	unsigned int prefix;
	unsigned int suffix;
	unsigned int shortest;

	shortest = st_min2(l1, l2);
	prefix = st_longest_common_prefix(s1, l1, s2, l2, shortest);
	suffix = st_longest_common_suffix(s1, l1, s2, l2, shortest);

	return st_max2(l1, l2) - st_min3(prefix + suffix, l1, l2);
}
