#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "libs/xmalloc.h"

static unsigned int
st_min3(unsigned int a, unsigned int b, unsigned int c)
{
	unsigned int n;

	n = a;
	if(b < n)
		n = b;
	if(c < n)
		n = c;

	return n;
}

unsigned int
levenshtein(const char *s, unsigned long int sl,
	const char *t, unsigned long int tl)
{
	unsigned int i, j;
	unsigned int cost;
	unsigned int *d;

	assert(s);
	assert(t);

	d = xcalloc((sl + 1) * (tl + 1), sizeof(unsigned int));

	d[(0) + (0) * sl] = 0;
	for(i = 0; i < sl; i++)
		d[(1 + i) + (0) * sl] = i + 1;
	for(j = 0; j < tl; j++)
		d[(0) + (1 + j) * sl] = j + 1;

	for(i = 0; i < sl; i++) {
		for(j = 0; j < tl; j++) {
			/* Case-insensitive */
			if(tolower(s[i]) == tolower(t[j])) {
				d[(1 + i) + (1 + j) * sl] =
					d[(0 + i) + (0 + j) * sl];
				continue;
			}

			d[(1 + i) + (1 + j) * sl] = st_min3(
				d[(0 + i) + (1 + j) * sl] + 1,
				d[(1 + i) + (0 + j) * sl] + 1,
				d[(0 + i) + (0 + j) * sl] + 1);
		}
	}

	cost = d[(sl) + (tl) * sl];
	free(d);

	return cost;
}
