#include <stdlib.h>
#include <string.h>

#include "libs/buffer.h"
#include "normalize-no.h"

/* XXX: This functions use strncmp without making sure that the string
 * is longer than the length of the string. */

/* XXX: Should really replace all of this with UTF-8 aware string
 * handling. Is this even possible? */

void
normalize_no(const char *s, unsigned int l,
	char **out, unsigned int *out_length)
{
	/* Declare the following string constants with []. This in order to
	 * get their length using sizeof() at compile-time instead of strlen
	 * at run-time. Is this portable? -Vegard */
	static const char aelig[] = "æ";
	static const char Aelig[] = "Æ";
	static const char oslash[] = "ø";
	static const char Oslash[] = "Ø";
	static const char aring[] = "å";
	static const char Aring[] = "Å";
	static unsigned int aelig_len = sizeof(aelig) - 1;
	static unsigned int Aelig_len = sizeof(Aelig) - 1;
	static unsigned int oslash_len = sizeof(oslash) - 1;
	static unsigned int Oslash_len = sizeof(Oslash) - 1;
	static unsigned int aring_len = sizeof(aring) - 1;
	static unsigned int Aring_len = sizeof(Aring) - 1;

	unsigned int i;
	struct buffer out_buffer;

	buffer_init(&out_buffer);

	i = 0;
	while(i < l) {
		/* Single-byte uppercase characters (ASCII) */
		if(s[i] >= 'A' && s[i] <= 'Z') {
			buffer_write_char(&out_buffer, s[i]);
			i++;
			continue;
		}

		/* Single-byte lowercase characters (ASCII) */
		if(s[i] >= 'a' && s[i] <= 'z') {
			char c = s[i] - 'a' + 'A';
			buffer_write_char(&out_buffer, c);
			i++;
			continue;
		}

		/* Single-byte numerics (ASCII) */
		if(s[i] >= '0' && s[i] <= '9') {
			buffer_write_char(&out_buffer, s[i]);
			i++;
			continue;
		}

		/* Æ */
		if(!strncmp(aelig, s + i, aelig_len) || !strncmp(Aelig, s + i, Aelig_len)) {
			buffer_write(&out_buffer, Aelig, Aelig_len);
			i += Aelig_len;
			continue;
		}

		/* Ø */
		if(!strncmp(oslash, s + i, oslash_len) || !strncmp(Oslash, s + i, Oslash_len)) {
			buffer_write(&out_buffer, Oslash, Oslash_len);
			i += Oslash_len;
			continue;
		}

		/* Å */
		if(!strncmp(aring, s + i, aring_len) || !strncmp(Aring, s + i, Aring_len)) {
			buffer_write(&out_buffer, Aring, Aring_len);
			i += Aring_len;
			continue;
		}

		/* Skip other characters */
		i++;
	}

	buffer_duplicate(&out_buffer, out, out_length);
	buffer_deinit(&out_buffer, 1);
	return;
}
