#ifndef _REQUEST_PARSER_H
#define _REQUEST_PARSER_H

#include <libxml/parser.h>

#include "libs/buffer.h"

struct request;

struct request_parser_handler {
	void (*on_start_document)(struct request *);
	void (*on_start_sources)(struct request *);
	void (*on_start_category)(struct request *);
	void (*on_start_thesaurus)(struct request *);
	void (*on_start_text)(struct request *);
	void (*on_start_word)(struct request *);

	void (*on_end_document)(struct request *);
	void (*on_end_sources)(struct request *);
	void (*on_end_category)(struct request *);
	void (*on_end_thesaurus)(struct request *);
	void (*on_end_text)(struct request *);
	void (*on_end_word)(struct request *);

	void (*on_characters_category)(
		struct request *,
		const char *,
		unsigned int
	);
	void (*on_characters_thesaurus)(
		struct request *,
		const char *,
		unsigned int
	);
	void (*on_characters_text)(
		struct request *,
		const char *,
		unsigned int
	);
	void (*on_characters_word)(
		struct request *,
		const char *,
		unsigned int
	);
};

struct request_parser {
	enum {
		REQUEST_PARSER_ROOT,
		REQUEST_PARSER_ROOT_DOCUMENT,
		REQUEST_PARSER_ROOT_DOCUMENT_SOURCES,
		REQUEST_PARSER_ROOT_DOCUMENT_SOURCES_CATEGORY,
		REQUEST_PARSER_ROOT_DOCUMENT_SOURCES_THESAURUS,
		REQUEST_PARSER_ROOT_DOCUMENT_TEXT,
		REQUEST_PARSER_ROOT_DOCUMENT_TEXT_WORD,
		REQUEST_PARSER_ERROR,
		REQUEST_PARSER_DONE,
	} state;

	/* error message (NULL if none) */
	char *error;

	struct request *request;
	const struct request_parser_handler *handler;
};

extern const xmlSAXHandler request_parser_handler;

void
request_parser_init(struct request_parser *parser);

void
request_parser_deinit(struct request_parser *parser);

#endif
