#ifndef _RESULT_H
#define _RESULT_H

#include "transformations.h"

struct result {
	struct result *_prev, *_next;

	struct synonym *synonym;

	/* This field helps determine the final ranking score
	 * when finding the best matches for a query. */
	struct transformations transformations;

	/* Which index the synonym was found in. */
	int index : 1;
	int index_a : 1;
	int index_b : 1;
	int index_ab : 1;

	/* Edit distance. A value of zero means an exact (best) match. */
	int distance;

	/* Score. Must be computed after the other fields have been set. Zero
	 * indicates an exact match; higher scores are worse. */
	unsigned int score;
};

void result_init(struct result *result);
void result_deinit(struct result *result);

/* Use the function as follows:
 * 
 * struct result **array;
 * unsigned int n;
 *
 * result_list_sort(some_list, &array, &n);
 *
 * for(i = 0; i < n; i++)
 *         printf("%d %.*s\n", array[i]->score,
 *                 array[i]->synonym->word,
 *                 array[i]->synonym->word_length);
 */
void result_list_sort(struct result *result_list,
	struct result ***result_array, unsigned int *result_n);

#endif
