#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "libs/buffer.h"
#include "libs/xmalloc.h"
#include "stemmer/libstemmer.h"

#include "stem-no.h"

/* Declare the following string constants with []. This in order to
 * get their length using sizeof() at compile-time instead of strlen
 * at run-time. Is this portable? -Vegard */
static const char aelig[] = "æ";
static const char Aelig[] = "Æ";
static const char oslash[] = "ø";
static const char Oslash[] = "Ø";
static const char aring[] = "å";
static const char Aring[] = "Å";
static unsigned int aelig_len = sizeof(aelig) - 1;
static unsigned int Aelig_len = sizeof(Aelig) - 1;
static unsigned int oslash_len = sizeof(oslash) - 1;
static unsigned int Oslash_len = sizeof(Oslash) - 1;
static unsigned int aring_len = sizeof(aring) - 1;
static unsigned int Aring_len = sizeof(Aring) - 1;

/* This was copied from normalize-no.c and modified. */
static void
st_uppercase(const char *s, unsigned int l,
	char **out, unsigned int *out_length)
{
	unsigned int i;
	struct buffer out_buffer;

	buffer_init(&out_buffer);

	i = 0;
	while(i < l) {
		/* Single-byte lowercase characters (ASCII) */
		if(s[i] >= 'a' && s[i] <= 'z') {
			char c = toupper(s[i]);
			buffer_write(&out_buffer, &c, 1);
			i++;
			continue;
		}

		/* Æ */
		if(!strncmp(aelig, s + i, aelig_len) || !strncmp(Aelig, s + i, Aelig_len)) {
			buffer_write(&out_buffer, Aelig, Aelig_len);
			i += Aelig_len;
			continue;
		}

		/* Ø */
		if(!strncmp(oslash, s + i, oslash_len) || !strncmp(Oslash, s + i, Oslash_len)) {
			buffer_write(&out_buffer, Oslash, Oslash_len);
			i += Oslash_len;
			continue;
		}

		/* Å */
		if(!strncmp(aring, s + i, aring_len) || !strncmp(Aring, s + i, Aring_len)) {
			buffer_write(&out_buffer, Aring, Aring_len);
			i += Aring_len;
			continue;
		}

		/* Add other characters */
		buffer_write(&out_buffer, &s[i], 1);
		i++;
	}

	buffer_duplicate(&out_buffer, out, out_length);
	buffer_deinit(&out_buffer, 1);
	return;
}

static void
st_lowercase(const char *s, unsigned int l,
	char **out, unsigned int *out_length)
{
	unsigned int i;
	struct buffer out_buffer;

	buffer_init(&out_buffer);

	i = 0;
	while(i < l) {
		/* Single-byte lowercase characters (ASCII) */
		if(s[i] >= 'A' && s[i] <= 'Z') {
			char c = tolower(s[i]);
			buffer_write(&out_buffer, &c, 1);
			i++;
			continue;
		}

		/* Æ */
		if(!strncmp(aelig, s + i, aelig_len) || !strncmp(Aelig, s + i, Aelig_len)) {
			buffer_write(&out_buffer, aelig, aelig_len);
			i += Aelig_len;
			continue;
		}

		/* Ø */
		if(!strncmp(oslash, s + i, oslash_len) || !strncmp(Oslash, s + i, Oslash_len)) {
			buffer_write(&out_buffer, oslash, oslash_len);
			i += Oslash_len;
			continue;
		}

		/* Å */
		if(!strncmp(aring, s + i, aring_len) || !strncmp(Aring, s + i, Aring_len)) {
			buffer_write(&out_buffer, aring, aring_len);
			i += Aring_len;
			continue;
		}

		/* Add other characters */
		buffer_write(&out_buffer, &s[i], 1);
		i++;
	}

	buffer_duplicate(&out_buffer, out, out_length);
	buffer_deinit(&out_buffer, 1);
}

/* XXX: Yes, I know. This is one ugly function. */
void
stem_no(const char *s, unsigned int l,
	char **out, unsigned int *out_length)
{
	struct sb_stemmer *stemmer;

	char *lowercase;
	unsigned int lowercase_length;

	const sb_symbol *stemmed;
	unsigned int stemmed_length;

	st_lowercase(s, l, &lowercase, &lowercase_length);

	stemmer = sb_stemmer_new("norwegian", "UTF_8");
	if(!stemmer) {
		fprintf(stderr, "error: could not initialize stemmer\n");
		exit(1);
	}

	stemmed = sb_stemmer_stem(stemmer, (sb_symbol *) lowercase, lowercase_length);
	stemmed_length = sb_stemmer_length(stemmer);

	free(lowercase);

	st_uppercase((char *) stemmed, stemmed_length, out, out_length);
	sb_stemmer_delete(stemmer);
}
