#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include "libs/xmalloc.h"

#include "bloom-filter.h"
#include "trie.h"
#include "synonym.h"
#include "synonym-array.h"
#include "thesaurus.h"

void
synonym_array_init(struct synonym_array *a)
{
	assert(a);

	a->n = 0;
	a->max = 0;

	a->synonyms = NULL;
}

void
synonym_array_deinit(struct synonym_array *a, unsigned int free_synonyms)
{
	unsigned int i;

	assert(a);

	if(free_synonyms) {
		for(i = 0; i < a->n; i++) {
			synonym_deinit(a->synonyms[i]);
			free(a->synonyms[i]);
		}
	}

	free(a->synonyms);
}

void
synonym_array_insert(struct synonym_array *a, struct synonym *synonym)
{
	assert(a);
	assert(synonym);

	if(a->n == a->max) {
		/* Double array size each time we run out of space. Start with 64 elements. */
		a->max = (a->max == 0) ? 64 : a->max * 2;
		a->synonyms = xrealloc(a->synonyms, a->max * sizeof(struct synonym *));
	}

	a->synonyms[a->n++] = synonym;
}

/* The array MUST contain the synonym before calling this function. */
void
synonym_array_delete(struct synonym_array *a, struct synonym *synonym)
{
	unsigned int i;

	assert(a);
	assert(synonym);

	for(i = 0; i < a->n; i++) {
		if(a->synonyms[i] == synonym)
			break;
	}

	/* Remove synonym but retain the allocated slot in the array */
	a->n -= 1;
	a->synonyms[i] = a->synonyms[a->n];
	a->synonyms[a->n] = NULL;
}
