#include <stdio.h>
#include <stdlib.h>

#include "libs/list.h"

#include "term.h"
#include "term-hashtable.h"

void
term_hashtable_init(struct term_hashtable *table)
{
	unsigned int i;

	for(i = 0; i < 256; i++)
		LIST_INIT(&table->buckets[i]);
}

void
term_hashtable_deinit(struct term_hashtable *table)
{
	unsigned int i;
	struct term *term;
	struct term *term_temp;

	for(i = 0; i < 256; i++) {
		LIST_FOR_SAFE(&table->buckets[i], term, term_temp) {
			LIST_REMOVE(term);
			term_deinit(term);
			free(term);
		}
	}
}

static unsigned int
st_hash(unsigned long int id)
{
	return id % 256;
}

struct term *
term_hashtable_get(struct term_hashtable *table,
	unsigned long int id)
{
	unsigned int i;
	struct term *bucket;
	struct term *term;

	i = st_hash(id);
	bucket = &table->buckets[i];

	LIST_FOR(bucket, term) {
		if(term->id == id)
			return term;
	}

	return NULL;
}

void
term_hashtable_put(struct term_hashtable *table,
	struct term *term)
{
	unsigned int i;
	struct term *bucket;

	i = st_hash(term->id);
	bucket = &table->buckets[i];

	LIST_INSERT_BEFORE(bucket, term);
}

#if 0
/* Simple hashtable profiler. Shows the number of elements in each bucket.
 * Useful for evaluating the hash function against data sets. */
void
term_hashtable_profile(struct term_hashtable *table)
{
	unsigned int i;
	unsigned int n;
	struct term *bucket;
	struct term *term;

	for(i = 0; i < 256; i++) {
		bucket = &table->buckets[i];

		n = 0;
		LIST_FOR(bucket, term)
			n++;

		printf("%u %u\n", i, n);
	}
}
#endif
