#include <stdio.h>
#include <stdlib.h>

#include "libs/list.h"

#include "thesaurus.h"
#include "thesaurus-hashtable.h"

void
thesaurus_hashtable_init(struct thesaurus_hashtable *table)
{
	unsigned int i;

	for(i = 0; i < 256; i++)
		LIST_INIT(&table->buckets[i]);
}

void
thesaurus_hashtable_deinit(struct thesaurus_hashtable *table)
{
	unsigned int i;
	struct thesaurus *thesaurus;
	struct thesaurus *thesaurus_temp;

	for(i = 0; i < 256; i++) {
		LIST_FOR_SAFE(&table->buckets[i], thesaurus, thesaurus_temp) {
			LIST_REMOVE(thesaurus);
			thesaurus_deinit(thesaurus);
			free(thesaurus);
		}
	}
}

static unsigned int
st_hash(unsigned long int id)
{
	return id % 256;
}

struct thesaurus *
thesaurus_hashtable_get(struct thesaurus_hashtable *table,
	unsigned long int id)
{
	unsigned int i;
	struct thesaurus *bucket;
	struct thesaurus *thesaurus;

	i = st_hash(id);
	bucket = &table->buckets[i];

	LIST_FOR(bucket, thesaurus) {
		if(thesaurus->id == id)
			return thesaurus;
	}

	return NULL;
}

void
thesaurus_hashtable_put(struct thesaurus_hashtable *table,
	struct thesaurus *thesaurus)
{
	unsigned int i;
	struct thesaurus *bucket;

	i = st_hash(thesaurus->id);
	bucket = &table->buckets[i];

	LIST_INSERT_BEFORE(bucket, thesaurus);
}
