#ifndef _THESAURUS_H
#define _THESAURUS_H

#include "bloom-filter.h"
#include "synonym-array.h"
#include "trie.h"

struct category;

struct thesaurus {
	struct thesaurus *_prev, *_next;

	unsigned long int id;
	struct category *category;
	char *name;
	unsigned int name_length;

	/* Index */
	struct synonym_array index;
	/* Indexed by first letter */
	struct synonym_array index_a[26];
	/* Indexed by last letter */
	struct synonym_array index_b[26];
	/* Indexed by first and last letter. Compute by first + 26 * last */
	struct synonym_array index_ab[26 * 26];
	/* Bloom filter structure */
	struct bloom_filter bloom_filter;
	/* Tree structure */
	struct trie trie;
};

void thesaurus_init(struct thesaurus *thesaurus);
void thesaurus_deinit(struct thesaurus *thesaurus);

void thesaurus_insert_term(struct thesaurus *thesaurus, struct term *term);
void thesaurus_insert_synonym(struct thesaurus *thesaurus, struct synonym *synonym);

struct result;

void thesaurus_search_trie(struct thesaurus *thesaurus,
	struct result *result_list,
	const char *s, unsigned int l,
	const struct transformations *t);

void thesaurus_search_index(struct thesaurus *thesaurus,
	struct result *result_list,
	const char *s, unsigned int l,
	const struct transformations *t);

void thesaurus_search_index_a(struct thesaurus *thesaurus,
	struct result *result_list,
	const char *s, unsigned int l,
	const struct transformations *t);

void thesaurus_search_index_b(struct thesaurus *thesaurus,
	struct result *result_list,
	const char *s, unsigned int l,
	const struct transformations *t);

void thesaurus_search_index_ab(struct thesaurus *thesaurus,
	struct result *result_list,
	const char *s, unsigned int l,
	const struct transformations *t);

#endif
