#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "libs/buffer.h"

#include "translate.h"

void
translate_prepare(struct translation *table)
{
	int i;

	for(i = 0; table[i].from; i++) {
		table[i].from_length = strlen(table[i].from);
		table[i].to_length = strlen(table[i].to);
	}
}

static const struct translation *
st_get_translation(struct translation *table, const char *s, unsigned int l)
{
	int i;

	for(i = 0; table[i].from; i++) {
		/* It can't be this one. */
		if(table[i].from_length > l)
			continue;

		if(!strncmp(table[i].from, s, table[i].from_length))
			return &table[i];
	}

	return NULL;
}

void
translate(struct translation *table,
	const char *s, unsigned int l,
	char **out, unsigned int *out_length,
	int keep_others)
{
	unsigned int i;
	const struct translation *tr;
	struct buffer out_buffer;

	buffer_init(&out_buffer);

	i = 0;
	while(i < l) {
		tr = st_get_translation(table, s + i, l - i);
		if(!tr) {
			buffer_write(&out_buffer, s + i, 1);
			i++;
			continue;
		}

		if(keep_others)
			buffer_write(&out_buffer, tr->to, tr->to_length);

		i += tr->from_length;
	}

	buffer_duplicate(&out_buffer, out, out_length);
	buffer_deinit(&out_buffer, 1);
	return;
}
